/*********************************************************************************
 * THE FOLLOWING RESOURCES ARE USED:                                             *
 *                                                                               *
 *     LCD:                                                                      *
 *         -Pins E3, E4, E5, E6, E7, E8, E9, E10, E11, E12                       *
 *                                                                               *
 *     Pushbuttons:                                                              *
 *         -Left    Pin A8,  TIM1                                                *
 *         -Right   Pin A2,  TIM9                                                *
 *         -Up      Pin B4,  TIM3                                                *
 *         -Down    Pin B14, TIM12                                               *
 *                                                                               *
 *     PWM:                                                                      *
 *         -Pin A1, TIM5                                                         *
 *                                                                               *
 *     Blinking Underline:                                                       *
 *         -TIM6                                                                 *
 *********************************************************************************/

#include "stm32f4xx.h"
#include "core_cm4.h"

#include "uc1608fb.h"
#include "interrupts.h"
#include "screens.h"

/*********************************************************************************
 * PROGRAM STATE                                                                 *
 *********************************************************************************/
enum Mode {SWEEP, THREE_POS, TWO_POS, ONE_POS};
enum SelectedOption	{MODE,
					SWEEP_START, SWEEP_STOP, SWEEP_CYCLES, SWEEP_RATE, SWEEP_STEP,
					THREEPOS_1POS, THREEPOS_2POS, THREEPOS_3POS, THREEPOS_RATE, THREEPOS_WAIT, THREEPOS_CYCLES,
					TWOPOS_1POS, TWOPOS_2POS, TWOPOS_RATE, TWOPOS_WAIT, TWOPOS_CYCLES,
					ONEPOS_POSITION, ONEPOS_RATE};

// Default state for each mode
volatile uint16_t sweep_start = 9000;				// in tenths of a microsecond.		valid range = 8000 - 22000 (800 - 2200us)
volatile uint16_t sweep_stop = 21000;				// in tenths of a microsecond.		valid range = 8000 - 22000 (800 - 2200us)
volatile uint32_t sweep_cycles = 0;					// 0 = no limit.					valid range = 0 - 9999999
volatile uint16_t sweep_rate = 10;					// framerate in milliseconds.		valud range = 3-30
volatile uint16_t sweep_step = 70;					// in tenths of a microsecond.		valid range = 1 - 9999 (0.1 - 999.9us)

volatile uint16_t threePos_firstPosition = 9000;	// in tenths of a microsecond.		valid range = 8000 - 22000 (800 - 2200us)
volatile uint16_t threePos_secondPosition = 15000;	// in tenths of a microsecond.		valid range = 8000 - 22000 (800 - 2200us)
volatile uint16_t threePos_thirdPosition = 21000;	// in tenths of a microsecond.		valid range = 8000 - 22000 (800 - 2200us)
volatile uint16_t threePos_rate = 10;				// framerate in milliseconds.		valud range = 3-30
volatile uint16_t threePos_wait = 10;				// in tenths of a second.			valid range = 1 - 999
volatile uint32_t threePos_cycles = 0;				// 0 = no limit.					valid range = 0 - 9999999

volatile uint16_t twoPos_firstPosition = 9000;		// in tenths of a microsecond.		valid range = 8000 - 22000 (800 - 2200us)
volatile uint16_t twoPos_secondPosition = 21000;	// in tenths of a microsecond.		valid range = 8000 - 22000 (800 - 2200us)
volatile uint16_t twoPos_rate = 10;					// framerate in milliseconds.		valud range = 3-30
volatile uint16_t twoPos_wait = 18;					// in tenths of a second.			valid range = 1 - 999
volatile uint32_t twoPos_cycles = 0;				// 0 = no limit.					valid range = 0 - 9999999

volatile uint16_t onePos_position = 15000;			// in tenths of a microsecond.		valid range = 8000 - 22000 (800 - 2200us)
volatile uint16_t onePos_rate = 10;					// framerate in milliseconds.		valud range = 3-30

// Defaults
volatile enum Mode currentMode = TWO_POS;
volatile enum SelectedOption currentlySelectedOption = MODE;
volatile uint32_t cyclesCompleted = 0;
volatile uint16_t currentServoPulse = 9000;			// in tenths of a microsecond.		valid range = 8000 - 22000 (800 - 2200us)
volatile uint8_t drawUnderline = 0;					// 0 or 1 to draw or not draw an underline below the currently selected item

/*********************************************************************************
 * MAIN                                                                          *
 *********************************************************************************/
int main() {
	lcd_init(E3, E4, E5, E6, E7, E8, E9, E10, E11, E12);	// CD, WR1, D7, D6, D5, D4, D3, D2, D1, D0
	//lcd_write_pixbuf();									// splash screen (shows the contents of pixBuf[][])
	//delay(1500000);										// 1.5 seconds
	
	RCC->AHB1ENR |= RCC_AHB1ENR_GPIOAEN;					// Enable GPIOA clock
	GPIOA->MODER |= GPIO_MODER_MODER8_1;					// Pin A8 = Alt Function		LEFT ARROW PUSHBUTTON
	GPIOA->AFR[1] |= 0b0001;								// AF1 for pin A2 (TIM1-2)		LEFT ARROW PUSHBUTTON
	GPIOA->MODER |= GPIO_MODER_MODER2_1;					// Pin A2 = Alt Function		RIGHT ARROW PUSHBUTTON
	GPIOA->AFR[0] |= 0b001100000000;						// AF3 for pin A2 (TIM8-11)		RIGHT ARROW PUSHBUTTON
	GPIOA->MODER |= GPIO_MODER_MODER1_1;					// Pin A1 = Alt Function		PWM
	GPIOA->AFR[0] |= 0b00100000;							// AF2 for pin A1 (TIM3-5)		PWM	
	
	RCC->AHB1ENR |= RCC_AHB1ENR_GPIOBEN;					// Enable GPIOB clock
	GPIOA->MODER |= GPIO_MODER_MODER4_1;					// Pin B4 = Alt Function		UP ARROW PUSHBUTTON
	GPIOA->AFR[0] |= 0b0010000000000000;					// AF2 for pin B4 (TIM3-5)		UP ARROW PUSHBUTTON
	GPIOA->MODER |= GPIO_MODER_MODER14_1;					// Pin B14 = Alt Function		DOWN ARROW PUSHBUTTON
	GPIOA->AFR[1] |= 0b1001000000000000000000000000;		// AF9 for pin B14 (TIM12-14)	DOWN ARROW PUSHBUTTON

	RCC->APB2ENR |= RCC_APB2ENR_TIM1EN;						// Enable clock					LEFT ARROW PUSHBUTTON
	TIM1->PSC = 41999;										// Prescaler = 41999
	TIM1->ARR = 999;										// Auto-reload = 999 (UIE every half second)
	TIM1->SMCR |= TIM_SMCR_TS_0 | TIM_SMCR_TS_2;			// TS=101 to trigger on TI1
	TIM1->SMCR |= TIM_SMCR_SMS_2;							// SMS=100 for reset on TI1
	TIM1->EGR |= TIM_EGR_UG;								// Force update
	TIM1->SR &= ~TIM_SR_UIF;								// Clear the update flag
	TIM1->DIER |= TIM_DIER_UIE;								// Enable interrupt on update event
	NVIC_EnableIRQ(TIM1_UP_TIM10_IRQn);						// Enable IRQ
	TIM1->CR1 |= TIM_CR1_CEN;								// Enable counter
	
	RCC->APB2ENR |= RCC_APB2ENR_TIM9EN;						// Enable clock					RIGHT ARROW PUSHBUTTON
	TIM9->PSC = 41999;										// Prescaler = 41999
	TIM9->ARR = 999;										// Auto-reload = 999 (UIE every half second)
	TIM9->SMCR |= TIM_SMCR_TS_0 | TIM_SMCR_TS_2;			// TS=101 to trigger on TI1
	TIM9->SMCR |= TIM_SMCR_SMS_2;							// SMS=100 for reset on TI1
	TIM9->EGR |= TIM_EGR_UG;								// Force update
	TIM9->SR &= ~TIM_SR_UIF;								// Clear the update flag
	TIM9->DIER |= TIM_DIER_UIE;								// Enable interrupt on update event
	NVIC_EnableIRQ(TIM1_BRK_TIM9_IRQn);						// Enable IRQ
	TIM9->CR1 |= TIM_CR1_CEN;								// Enable counter
	
	RCC->APB1ENR |= RCC_APB1ENR_TIM3EN;						// Enable clock					UP ARROW PUSHBUTTON
	TIM3->PSC = 10499;										// Prescaler = 10499
	TIM3->ARR = 999;										// Auto-reload = 999 (UIE every half second)
	TIM3->SMCR |= TIM_SMCR_TS_0 | TIM_SMCR_TS_2;			// TS=101 to trigger on TI1
	TIM3->SMCR |= TIM_SMCR_SMS_2;							// SMS=100 for reset on TI1
	TIM3->EGR |= TIM_EGR_UG;								// Force update
	TIM3->SR &= ~TIM_SR_UIF;								// Clear the update flag
	TIM3->DIER |= TIM_DIER_UIE;								// Enable interrupt on update event
	NVIC_EnableIRQ(TIM3_IRQn);								// Enable IRQ
	TIM3->CR1 |= TIM_CR1_CEN;								// Enable counter
	
	RCC->APB1ENR |= RCC_APB1ENR_TIM12EN;					// Enable clock					DOWN ARROW PUSHBUTTON
	TIM12->PSC = 10499;										// Prescaler = 10499
	TIM12->ARR = 999;										// Auto-reload = 999 (UIE every half second)
	TIM12->SMCR |= TIM_SMCR_TS_0 | TIM_SMCR_TS_2;			// TS=101 to trigger on TI1
	TIM12->SMCR |= TIM_SMCR_SMS_2;							// SMS=100 for reset on TI1
	TIM12->EGR |= TIM_EGR_UG;								// Force update
	TIM12->SR &= ~TIM_SR_UIF;								// Clear the update flag
	TIM12->DIER |= TIM_DIER_UIE;							// Enable interrupt on update event
	NVIC_EnableIRQ(TIM8_BRK_TIM12_IRQn);					// Enable IRQ
	TIM12->CR1 |= TIM_CR1_CEN;								// Enable counter
	
	RCC->APB1ENR |= RCC_APB1ENR_TIM6EN;						// Enable TIM6 clock			BLINKING UNDERLINE
	TIM6->PSC = 41999;										// Set prescaler to 41999
	TIM6->ARR = 799;										// Set auto-reload to 799
	TIM6->EGR |= TIM_EGR_UG;								// Force update
	TIM6->SR &= ~TIM_SR_UIF;								// Clear the update flag
	TIM6->DIER |= TIM_DIER_UIE;								// Enable interrupt on update event
	NVIC_EnableIRQ(TIM6_DAC_IRQn);							// Enable TIM6 IRQ
	TIM6->CR1 |= TIM_CR1_CEN;								// Enable TIM6 counter
	
	RCC->APB1ENR |= RCC_APB1ENR_TIM5EN;						// Enable clock					PWM
	TIM5->PSC = 41;											// Prescaler = 42 (1 tick per half microsecond)
	TIM5->ARR = (sweep_rate * 2000) - 1;					// Auto-reload = PWM frequency in half microseconds
	TIM5->CCMR1 |= TIM_CCMR1_OC2M_1 | TIM_CCMR1_OC2M_2;		// OC2M = 110 for PWM Mode 1 output on ch2
	TIM5->CCMR1 |= TIM_CCMR1_OC2PE;							// Output 2 preload enable
	TIM5->CR1 |= TIM_CR1_ARPE;								// Auto-reload preload enable
	TIM5->CCER |= TIM_CCER_CC2E;							// Enable output for ch2
	TIM5->CCR2 = currentServoPulse / 5;						// CCR2 = Duty cycle in half microseconds
	TIM5->EGR |= TIM_EGR_UG;								// Force update
	TIM5->SR &= ~TIM_SR_UIF;								// Clear the update flag
	TIM5->DIER |= TIM_DIER_UIE;								// Enable interrupt on update event
	NVIC_EnableIRQ(TIM5_IRQn);								// Enable IRQ
	TIM5->CR1 |= TIM_CR1_CEN;								// Enable counter
	
	// Draw screen based on current state
	while(1) {
		if(currentMode == SWEEP)
			drawSweepScreen();
		else if(currentMode == THREE_POS)
			draw3posScreen();
		else if(currentMode == TWO_POS)
			draw2posScreen();
		else if(currentMode == ONE_POS)
			draw1posScreen();
		
		delay(10000); // 0.01s
	}
}
